;
; Definition file for the DLL version of the SSLEAY library from OpenSSL
;

LIBRARY         SSLEAY32	

EXPORTS
    BIO_f_ssl                               @121
    BIO_new_buffer_ssl_connect              @173
    BIO_new_ssl                             @122
    BIO_new_ssl_connect                     @174
    BIO_ssl_copy_session_id                 @124
    BIO_ssl_shutdown                        @131
    DTLSv1_client_method                    @268
    DTLSv1_method                           @273
    DTLSv1_server_method                    @275
    ERR_load_SSL_strings                    @1
    SSL_CIPHER_description                  @2
    SSL_CIPHER_get_bits                     @128
    SSL_CIPHER_get_name                     @130
    SSL_CIPHER_get_version                  @129
    SSL_COMP_add_compression_method         @184
    SSL_COMP_get_compression_methods        @276
    SSL_COMP_get_name                       @271
    SSL_CTX_add_client_CA                   @3
    SSL_CTX_add_session                     @4
    SSL_CTX_callback_ctrl                   @243
    SSL_CTX_check_private_key               @5
    SSL_CTX_ctrl                            @6
    SSL_CTX_flush_sessions                  @7
    SSL_CTX_free                            @8
    SSL_CTX_get_cert_store                  @180
    SSL_CTX_get_client_CA_list              @9
    SSL_CTX_get_client_cert_cb              @288
    SSL_CTX_get_ex_data                     @138
    SSL_CTX_get_ex_new_index                @167
    SSL_CTX_get_info_callback               @282
    SSL_CTX_get_quiet_shutdown              @140
    SSL_CTX_get_timeout                     @179
    SSL_CTX_get_verify_callback             @10
    SSL_CTX_get_verify_depth                @228
    SSL_CTX_get_verify_mode                 @11
    SSL_CTX_load_verify_locations           @141
    SSL_CTX_new                             @12
    SSL_CTX_remove_session                  @13
    SSL_CTX_sess_get_get_cb                 @279
    SSL_CTX_sess_get_new_cb                 @287
    SSL_CTX_sess_get_remove_cb              @289
    SSL_CTX_sess_set_get_cb                 @280
    SSL_CTX_sess_set_new_cb                 @278
    SSL_CTX_sess_set_remove_cb              @285
    SSL_CTX_sessions                        @245
    SSL_CTX_set_cert_store                  @181
    SSL_CTX_set_cert_verify_callback        @232
    SSL_CTX_set_cipher_list                 @15
    SSL_CTX_set_client_CA_list              @16
    SSL_CTX_set_client_cert_cb              @284
    SSL_CTX_set_client_cert_engine          @293
    SSL_CTX_set_cookie_generate_cb          @283
    SSL_CTX_set_cookie_verify_cb            @281
    SSL_CTX_set_default_passwd_cb           @17
    SSL_CTX_set_default_passwd_cb_userdata  @235
    SSL_CTX_set_default_verify_paths        @142
    SSL_CTX_set_ex_data                     @143
    SSL_CTX_set_generate_session_id         @264
    SSL_CTX_set_info_callback               @286
    SSL_CTX_set_msg_callback                @266
    SSL_CTX_set_purpose                     @238
    SSL_CTX_set_quiet_shutdown              @145
    SSL_CTX_set_session_id_context          @231
    SSL_CTX_set_ssl_version                 @19
    SSL_CTX_set_timeout                     @178
    SSL_CTX_set_tmp_dh_callback             @176
    SSL_CTX_set_tmp_ecdh_callback           @269
    SSL_CTX_set_tmp_rsa_callback            @177
    SSL_CTX_set_trust                       @237
    SSL_CTX_set_verify                      @21
    SSL_CTX_set_verify_depth                @225
    SSL_CTX_use_PrivateKey                  @22
    SSL_CTX_use_PrivateKey_ASN1             @23
    SSL_CTX_use_PrivateKey_file             @24
    SSL_CTX_use_RSAPrivateKey               @25
    SSL_CTX_use_RSAPrivateKey_ASN1          @26
    SSL_CTX_use_RSAPrivateKey_file          @27
    SSL_CTX_use_certificate                 @28
    SSL_CTX_use_certificate_ASN1            @29
    SSL_CTX_use_certificate_chain_file      @222
    SSL_CTX_use_certificate_file            @30
    SSL_SESSION_cmp                         @132
    SSL_SESSION_free                        @31
    SSL_SESSION_get_ex_data                 @146
    SSL_SESSION_get_ex_new_index            @168
    SSL_SESSION_get_id                      @277
    SSL_SESSION_get_time                    @134
    SSL_SESSION_get_timeout                 @136
    SSL_SESSION_hash                        @133
    SSL_SESSION_new                         @32
    SSL_SESSION_print                       @33
    SSL_SESSION_print_fp                    @34
    SSL_SESSION_set_ex_data                 @148
    SSL_SESSION_set_time                    @135
    SSL_SESSION_set_timeout                 @137
    SSL_accept                              @35
    SSL_add_client_CA                       @36
    SSL_add_dir_cert_subjects_to_stack      @188
    SSL_add_file_cert_subjects_to_stack     @185
    SSL_alert_desc_string                   @37
    SSL_alert_desc_string_long              @38
    SSL_alert_type_string                   @39
    SSL_alert_type_string_long              @40
    SSL_callback_ctrl                       @244
    SSL_check_private_key                   @41
    SSL_clear                               @42
    SSL_connect                             @43
    SSL_copy_session_id                     @44
    SSL_ctrl                                @45
    SSL_do_handshake                        @125
    SSL_dup                                 @46
    SSL_dup_CA_list                         @47
    SSL_free                                @48
    SSL_get1_session                        @242
    SSL_get_SSL_CTX                         @150
    SSL_get_certificate                     @49
    SSL_get_cipher_list                     @52
    SSL_get_ciphers                         @55
    SSL_get_client_CA_list                  @56
    SSL_get_current_cipher                  @127
    SSL_get_current_compression             @272
    SSL_get_current_expansion               @274
    SSL_get_default_timeout                 @57
    SSL_get_error                           @58
    SSL_get_ex_data                         @151
    SSL_get_ex_data_X509_STORE_CTX_idx      @175
    SSL_get_ex_new_index                    @169
    SSL_get_fd                              @59
    SSL_get_finished                        @240
    SSL_get_info_callback                   @165
    SSL_get_peer_cert_chain                 @60
    SSL_get_peer_certificate                @61
    SSL_get_peer_finished                   @241
    SSL_get_privatekey                      @126
    SSL_get_quiet_shutdown                  @153
    SSL_get_rbio                            @63
    SSL_get_read_ahead                      @64
    SSL_get_rfd                             @246
    SSL_get_servername                      @291
    SSL_get_servername_type                 @292
    SSL_get_session                         @154
    SSL_get_shared_ciphers                  @65
    SSL_get_shutdown                        @155
    SSL_get_ssl_method                      @66
    SSL_get_verify_callback                 @69
    SSL_get_verify_depth                    @229
    SSL_get_verify_mode                     @70
    SSL_get_verify_result                   @157
    SSL_get_version                         @71
    SSL_get_wbio                            @72
    SSL_get_wfd                             @247
    SSL_has_matching_session_id             @249
    SSL_library_init                        @183
    SSL_load_client_CA_file                 @73
    SSL_load_error_strings                  @74
    SSL_new                                 @75
    SSL_peek                                @76
    SSL_pending                             @77
    SSL_read                                @78
    SSL_renegotiate                         @79
    SSL_renegotiate_pending                 @265
    SSL_rstate_string                       @80
    SSL_rstate_string_long                  @81
    SSL_set_SSL_CTX                         @290
    SSL_set_accept_state                    @82
    SSL_set_bio                             @83
    SSL_set_cipher_list                     @84
    SSL_set_client_CA_list                  @85
    SSL_set_connect_state                   @86
    SSL_set_ex_data                         @158
    SSL_set_fd                              @87
    SSL_set_generate_session_id             @258
    SSL_set_info_callback                   @160
    SSL_set_msg_callback                    @267
    SSL_set_purpose                         @236
    SSL_set_quiet_shutdown                  @161
    SSL_set_read_ahead                      @88
    SSL_set_rfd                             @89
    SSL_set_session                         @90
    SSL_set_session_id_context              @189
    SSL_set_shutdown                        @162
    SSL_set_ssl_method                      @91
    SSL_set_tmp_dh_callback                 @187
    SSL_set_tmp_ecdh_callback               @270
    SSL_set_tmp_rsa_callback                @186
    SSL_set_trust                           @239
    SSL_set_verify                          @94
    SSL_set_verify_depth                    @226
    SSL_set_verify_result                   @163
    SSL_set_wfd                             @95
    SSL_shutdown                            @96
    SSL_state                               @166
    SSL_state_string                        @97
    SSL_state_string_long                   @98
    SSL_use_PrivateKey                      @99
    SSL_use_PrivateKey_ASN1                 @100
    SSL_use_PrivateKey_file                 @101
    SSL_use_RSAPrivateKey                   @102
    SSL_use_RSAPrivateKey_ASN1              @103
    SSL_use_RSAPrivateKey_file              @104
    SSL_use_certificate                     @105
    SSL_use_certificate_ASN1                @106
    SSL_use_certificate_file                @107
    SSL_version                             @164
    SSL_want                                @182
    SSL_write                               @108
    SSLv23_client_method                    @110
    SSLv23_method                           @111
    SSLv23_server_method                    @112
    SSLv2_client_method                     @113
    SSLv2_method                            @114
    SSLv2_server_method                     @115
    SSLv3_client_method                     @116
    SSLv3_method                            @117
    SSLv3_server_method                     @118
    TLSv1_client_method                     @172
    TLSv1_method                            @170
    TLSv1_server_method                     @171
    d2i_SSL_SESSION                         @119
    i2d_SSL_SESSION                         @120

