#include <winver.h>

LANGUAGE 0x09,0x01

1 VERSIONINFO
  FILEVERSION 0,9,8,11
  PRODUCTVERSION 0,9,8,11
  FILEFLAGSMASK 0x3fL
#ifdef _DEBUG
  FILEFLAGS 0x01L
#else
  FILEFLAGS 0x00L
#endif
  FILEOS VOS__WINDOWS32
  FILETYPE VFT_DLL
  FILESUBTYPE 0x0L
BEGIN
    BLOCK "StringFileInfo"
    BEGIN
	BLOCK "040904b0"
	BEGIN
#if defined(FIPS)
	    VALUE "Comments", "WARNING: TEST VERSION ONLY ***NOT*** FIPS 140-2 VALIDATED.\0"
#endif
	    // Required:	    
	    VALUE "CompanyName", "The OpenSSL Project, http://www.openssl.org/\0"
#if defined(FIPS)
	    VALUE "FileDescription", "TEST UNVALIDATED FIPS140-2 DLL\0"
#else
	    VALUE "FileDescription", "OpenSSL Shared Library\0"
#endif
	    VALUE "FileVersion", "0.9.8k\0"
#if defined(CRYPTO)
	    VALUE "InternalName", "libeay32\0"
	    VALUE "OriginalFilename", "libeay32.dll\0"
#elif defined(SSL)
	    VALUE "InternalName", "ssleay32\0"
	    VALUE "OriginalFilename", "ssleay32.dll\0"
#elif defined(FIPS)
	    VALUE "InternalName", "libosslfips\0"
	    VALUE "OriginalFilename", "libosslfips.dll\0"
#endif
	    VALUE "ProductName", "The OpenSSL Toolkit\0"
	    VALUE "ProductVersion", "0.9.8k\0"
	    // Optional:
	    //VALUE "Comments", "\0"
	    VALUE "LegalCopyright", "Copyright © 1998-2007 The OpenSSL Project. Copyright © 1995-1998 Eric A. Young, Tim J. Hudson. All rights reserved.\0"
	    //VALUE "LegalTrademarks", "\0"
	    //VALUE "PrivateBuild", "\0"
	    //VALUE "SpecialBuild", "\0"
	END
    END
    BLOCK "VarFileInfo"
    BEGIN
        VALUE "Translation", 0x409, 0x4b0
    END
END
